//############################################################################
//
// LaserBoy !!!
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
// Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009 James Lehman.
// This source is distributed under the terms of the GNU General Public License.
//
// LaserBoy_palette.cpp is part of LaserBoy.
//
// LaserBoy is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// LaserBoy is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LaserBoy. If not, see <http://www.gnu.org/licenses/>.
//
//############################################################################
#include "LaserBoy_space.hpp"

//############################################################################
LaserBoy_palette& LaserBoy_palette::reorder()
{
    if(size())
    {
        LaserBoy_palette reordered(p_space);
        reordered.reserve(size());
        sort(begin(), end()); // sort this
        reordered.push_back(at(0));
        for(int i = 1; i < number_of_colors(); i++)
            if(at(i) != reordered.back()) // remove duplicates!
                reordered.push_back(at(i));
        *this = reordered;
    }
    return *this;
}

//############################################################################
int LaserBoy_palette::from_ifstream_ild(ifstream& in, const LaserBoy_header& header)
{
    int             i,
                    error = LASERBOY_OK;
    LaserBoy_color  color;
    //------------------------------------------------------------------------
    clear();
    reserve(header.quantity);
    //------------------------------------------------------------------------
    for(i = 0; i < header.quantity; i++)
    {
        if((error = color.from_ifstream_ild(in)))
            break;
        push_back(color);
    }
    find_factors();
    name = header.name;
    return error;
}

//############################################################################
void LaserBoy_palette::to_ofstream_ild(ofstream& out, LaserBoy_header& header) const
{
    int i;
    header.quantity = (u_short)number_of_colors();
    header.to_ofstream_ild(out);
    for(i = 0; i < number_of_colors(); i++)
        at(i).to_ofstream_ild(out);
    return;
}

//############################################################################
bool LaserBoy_palette::save_as_txt(string file) const
{
    ofstream out(file.c_str(), ios::out | ios::binary);

    if(out.is_open())
    {
        to_ofstream_txt(out);
        out.close();
    }
    else
        return false;
        
    return true;
}

//############################################################################
void LaserBoy_palette::to_ofstream_txt(ofstream& out) const
{
    int i;
    out << (   (number_of_colors() > 256)
             ? ("table")
             : ("palette")
           )
        << " rgb"
        << endl;

    for(i = 0; i < number_of_colors(); i++)
        at(i).to_ofstream_txt(out);
    out << endl;
    return;
}

//############################################################################
int LaserBoy_palette::from_ascii_file(string file)
{
    int             color_count = 0,
                    color_value,
                    error = LASERBOY_OK;
    LaserBoy_color  color;
    string          junk;
    //------------------------------------------------------------------------
    ifstream     in((LASERBOY_PALETTE_SHARE + file).c_str(), ios::in);
    if(in.is_open())
    {
        in >> name;
        in >> junk; // Red
        in >> junk; // Green
        in >> junk; // Blue
        reserve(LASERBOY_PALETTE_MAX);
        while(!in.eof() && (color_count < LASERBOY_PALETTE_MAX))
        {
            in >> color_value;    color.r = (char)color_value;
            in >> color_value;    color.g = (char)color_value;
            in >> color_value;    color.b = (char)color_value;
            push_back(color);
            color_count++;
        }
        in.close();
        find_factors();
    }
    else
        error = LASERBOY_FILE_OPEN_FAILED;
    return error;
}

//############################################################################
void LaserBoy_palette::save_as_ascii(string file) const
{
    ofstream out(file.c_str(), ios::out);
    if(out.is_open())
    {
        out << name
            << endl
            << "Red     Green   Blue"
            << endl;
        for(int i = 0; i < number_of_colors(); i++)
            out << setw(3)
                << (int)at(i).r
                << setw(4)
                << (int)at(i).g
                << setw(4)
                << (int)at(i).b
                << endl;
        out << endl;
        out.close();
    }
    return;
}

//############################################################################
int LaserBoy_palette::to_bmp_palette(struct LaserBoy_bmp* bmp) const
{
    int i;
    if(bmp->bpp == 8)
    {
        for(i = 0; i < LASERBOY_PALETTE_MAX; i++)
            if(i < number_of_colors())
                bmp_set_palette_index(bmp, i, at(i).r, at(i).g, at(i).b);
            else
                bmp_set_palette_index(bmp, i, 0, 0, 0);
        bmp->black = black;
        bmp->white = white;
        return LASERBOY_OK;
    }
    return LASERBOY_INCOMPATIBLE_FORMATS;
}

//############################################################################
void LaserBoy_palette::find_factors()
{
    int    i      ,
           j      ,
           k = 255;
    //------------------------------------------------------------------------
    first = black = white = last = 0;
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
    {
        if(at(black) > at(i))
          black = i;
        //--------------------------------------------------------------------
        if(at(white) < at(i))
          white = i;
        //--------------------------------------------------------------------
        j =   abs(at(i).r - 0x80)
            + abs(at(i).g - 0x80)
            + abs(at(i).b - 0x80);
        if(j < k)
            k    = j;
    }
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
        if(at(i).as_uint() > 0)
        {
            first = i;
            break;
        }
    //------------------------------------------------------------------------
    for(i = number_of_colors() - 1; i >= 0 ; i--)
        if(at(i).as_uint() > 0)
        {
            last = i;
            break;
        }
    //------------------------------------------------------------------------
    return;
}

//############################################################################
int LaserBoy_palette::best_match(LaserBoy_color rgb)
{
    int    i          ,
           j          ,
           k     = 255,
           match = 0  ;
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_colors(); i++)
    {
        j =   abs(at(i).r - rgb.r)
            + abs(at(i).g - rgb.g)
            + abs(at(i).b - rgb.b);
        if(j < k)
        {
            match = i;
            k     = j;
        }
    }
    //------------------------------------------------------------------------

    if(    p_space->no_black_match
        && (match == black)
      )
        match = white;
    //------------------------------------------------------------------------
    return match;
}

//############################################################################
void LaserBoy_palette::best_reduction() // reduces a color_table to a palette
{
    int    i          ,
           j          ,
           k     = 255,
           match = 0  ;
    //------------------------------------------------------------------------
    reorder();
    //------------------------------------------------------------------------
    if(number_of_colors() >= LASERBOY_PALETTE_MAX)
    {
        for(k = 1; k < 765; k++) // minimum difference
        {
            for(i = 0; i < number_of_colors() - 1; i++)
            {
                j =   abs(at(i).r - at(i + 1).r)
                    + abs(at(i).g - at(i + 1).g)
                    + abs(at(i).b - at(i + 1).b);
                if(j <= k)
                    erase(begin() + i + 1);
            }
            if(number_of_colors() <= LASERBOY_PALETTE_MAX)
                break;
        }
    }
    //------------------------------------------------------------------------
    find_factors();
    return;
}

//############################################################################
void LaserBoy_palette::shade(u_char shade) // 0 shade is no change 255 is black
{
    if(shade)
    {
        for(int i = 0; i < number_of_colors(); i++)
        {
            (at(i).r - shade > 0) ? (at(i).r -= shade) : (at(i).r = 0);
            (at(i).g - shade > 0) ? (at(i).g -= shade) : (at(i).g = 0);
            (at(i).b - shade > 0) ? (at(i).b -= shade) : (at(i).b = 0);
        }
    }
    return;
}

//############################################################################
void LaserBoy_palette::tint(u_char tint) // 0 tint is no change 255 is white
{
    if(tint)
    {
        for(int i = 0; i < number_of_colors(); i++)
        {
            (at(i).r + tint < 255) ? (at(i).r += tint) : (at(i).r = 255);
            (at(i).g + tint < 255) ? (at(i).g += tint) : (at(i).g = 255);
            (at(i).b + tint < 255) ? (at(i).b += tint) : (at(i).b = 255);
        }
    }
    return;
}

//############################################################################
bool LaserBoy_palette::unite(const LaserBoy_palette& palette)
{
    if(    palette.number_of_colors() >  0
        && palette.number_of_colors() <= LASERBOY_PALETTE_MAX
      )
    {
        int            i,
                       j;
        LaserBoy_palette     super_palette(*this);
        //--------------------------------------------------------------------
        super_palette.reserve(LASERBOY_PALETTE_MAX);
        //--------------------------------------------------------------------
        for(i = 0; i < palette.number_of_colors(); i++)
        {
            for(j = 0; j < super_palette.number_of_colors(); j++)
                if(super_palette[j] == palette.at(i))
                    break;
            if(j == super_palette.number_of_colors())
                super_palette.push_back(palette.at(i));
        }
        //--------------------------------------------------------------------
        if(super_palette.number_of_colors() <= LASERBOY_PALETTE_MAX)
        {
            super_palette.reorder();
            super_palette.find_factors();
            *this = super_palette;
            return true;
        }
        //--------------------------------------------------------------------
        else
            return false;
    }
    //------------------------------------------------------------------------
    return false;
}

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
