//############################################################################
//
// LaserBoy !!!
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
// Copyright 2003, 2004, 2005, 2006, 2007, 2008, 2009 James Lehman.
// This source is distributed under the terms of the GNU General Public License.
//
// LaserBoy_space.cpp is part of LaserBoy.
//
// LaserBoy is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// LaserBoy is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with LaserBoy. If not, see <http://www.gnu.org/licenses/>.
//
//############################################################################
#include "LaserBoy_space.hpp"

//############################################################################
LaserBoy_space::LaserBoy_space(LaserBoy_GUI_base* gui)
            : LaserBoy_frame_set      (this      )
            , LaserBoy_palette_set    (this      )
            , show_vertices           (false     ) // bool
            , show_blanking           (false     )
            , show_cursors            (false     )
            , show_fixed_bounds       (true      )
            , show_fixed_origin       (false     )
            , show_floating_axis      (false     )
            , show_floating_bounds    (false     )
            , show_fulcrum            (false     )
            , show_effects_generation (true      )
            , show_background_bitmap  (true      )
            , show_menu               (true      )
            , show_palettes           (true      )
            , show_stats              (true      )
            , show_TUI_clue           (true      )
            , show_frame_stats        (false     )
            , show_frame_set_stats    (false     )
            , ui_visuals_flip_flop    (true      )
            , loop_animation          (true      )
            , approximate_frame_rate  (true      )
            , wave_show_vertices      (false     )
            , wave_show_blanks        (false     )
            , wave_real_time          (true      )
            , wave_show_inverted      (false     )
            , wave_clean_screen       (true      )
            , wave_loop               (false     )
            , auto_scale_dxf          (true      )
            , maintain_dxf_origin     (false     )
            , black_dwell_vertices    (true      )
            , auto_flatten_z          (false     )
            , no_equivalent_vectors   (false     )
            , no_dots                 (false     )
            , invert_wave_output      (false     )
            , auto_optimize           (true      )
            , no_black_match          (false     )
            , allow_lit_black         (false     )
            , dumb_file_format        (false     )
            , save_txt_rgb            (true      )
            , save_txt_unit           (false     )
            , black_threshold         (0         ) // u_char
            , sample_rate             (48000     ) // int Hz
            , lit_dwell_overhang      (2         ) // samples
            , dwell_on_dot            (10        ) // samples
            , output_bmp_size         (1000      ) // pixels square
            , font_size_factor        (2         )
            , signal_if_not_z         (LASERBOY_SIGNAL_MONO_AVG_ANALOG)
            , unframed_wave_stride    (4800      ) // samples 1/10 of sec
            , displacement_step       (100       ) // pxls
            , lit_delta_max           (600       ) // double vertices
            , blank_delta_max         (900       ) // vertices
            , max_dwell_microsec      (333       )
            , insignificant_distance  (60        ) // vertices
            , insignificant_angle     (one_degree) // radians
            , recolor_span_factor     (2.0       )
            , rendered_arc_angle      (6.0       ) // degrees
            , frames_per_second       (16.0      ) // 0 == every frame is shown exactly once
            , still_frame_duration    (10.0      ) // seconds
            , rotation_step           (one_degree) // radians
            , magnitude_step          (0.01      )
            , magnitude_step_up       (1.01      )
            , magnitude_step_dn       (0.99      )
            , bmp                     (          ) // LaserBoy_bmp
            , background              (          )
            , background_bmp          (          )
            , bg_file_name            ("."       ) // string
            , view_scale              (1.0       ) // LaserBoy_3D_double
            , p_space                 (this      )
            , GUI                     (gui       )
{
    for(int i = 0; i < 8; i++)
    {
        wave_offsets   [i] =  0;
        signal_polarity[i] = -1;
    }
    signal_bit_mask[0] = 0;
    signal_bit_mask[1] = 0;
    signal_bit_mask[2] = 7;
    signal_bit_mask[3] = 7;
    signal_bit_mask[4] = 7;
    signal_bit_mask[5] = 0;
    signal_bit_mask[6] = 0;
    signal_bit_mask[7] = 0;

    load_cfg_file          ();
    load_frame_effects     ();
    load_frame_set_effects ();
}

//############################################################################
void LaserBoy_space::clear_unused_palettes()
{
    int              i,
                     j;
    LaserBoy_palette_set   palettes_in_use;
    //------------------------------------------------------------------------
    for(i = LASERBOY_ILDA_DEFAULT; i < LASERBOY_BUILT_IN_PALETTES; i++)
    {
        palette_picker(i).index  = i;
        palette_picker(i).in_use = true;
        palettes_in_use += palette_picker(i);
    }
    //------------------------------------------------------------------------
    for(i = LASERBOY_BUILT_IN_PALETTES; i < number_of_palettes(); i++)
    {
        palette_picker(i).index  = i;
        palette_picker(i).in_use = false;
    }
    //------------------------------------------------------------------------
    for(j = 0; j < number_of_frames(); j++)
        palette_picker(frame_picker(j).palette_index).in_use = true;
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_palettes(); i++)
    {
        if(palette_picker(i).in_use)
            palettes_in_use += palette_picker(i);
    }
    //------------------------------------------------------------------------
    for(i = 0; i < palettes_in_use.number_of_palettes(); i++)
    {
        for(j = 0; j < number_of_frames(); j++)
        {
            if(frame_picker(j).palette_index == palettes_in_use[i].index)
                frame_picker(j).palette_index = i;
        }
    }
    replace_palette_set(palettes_in_use);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void LaserBoy_space::sync_rgb_and_palette()
{
    int                   i                         = 0          ,
                          j                                      ,
                          super_palette_first_frame = 0          ,
                          super_palette_last_frame  = 0          ,
                          current_frame_index       = frame_index;
    LaserBoy_palette_set  reduced_palette_set (this)             ;
    LaserBoy_palette      super_palette       (this)             ;
    char                  super_palette_name[7]                  ;

    for(j = 0; j < number_of_frames(); j++)
        frame(j).sync_rgb_and_palette();
    //------------------------------------------------------------------------
    while(i < number_of_frames())
    {
        if(const_frame_picker(i).palette_index != LASERBOY_TRUE_COLOR)
        {
            //----------------------------------------------------------------
            super_palette_first_frame = i;
            super_palette = palette_picker(const_frame_picker(i).palette_index);
            sprintf(super_palette_name, "%06d", new_super_palette_id);
            super_palette.name = string("SP") + super_palette_name;
            new_super_palette_id++;
            //----------------------------------------------------------------
            while(    i < number_of_frames()
                   && super_palette.unite(palette_picker(const_frame_picker(i).palette_index))
                 )
            i++;
            //----------------------------------------------------------------
            reduced_palette_set.push_back(super_palette);
            super_palette_last_frame  = i - 1;
            //----------------------------------------------------------------
            for(j = super_palette_first_frame; j <= super_palette_last_frame; j++)
            {
                frame(j).find_rgb_in_palette(super_palette);
                frame(j).palette_index = reduced_palette_set.number_of_palettes() - 1;
            }
            //----------------------------------------------------------------
        }
        else
            i++;
    }
    //------------------------------------------------------------------------
    frame(current_frame_index);
    replace_palette_set(reduced_palette_set);
    palette_index = current_frame().palette_index;
    return;
}

//############################################################################
void LaserBoy_space::best_reduce_all_to_palette()
{
    int               i = 0;
    LaserBoy_palette  super_palette(this);
    char              super_palette_name[7];
    //------------------------------------------------------------------------
    for(i = 0; i < number_of_frames(); i++)
    {
        if(const_frame_picker(i).palette_index == LASERBOY_TRUE_COLOR)
            const_frame_picker(i).best_reduce_to_palette();

        super_palette += palette_picker(const_frame_picker(i).palette_index);
    }
    //------------------------------------------------------------------------
    super_palette.best_reduction();
    //------------------------------------------------------------------------
    sprintf(super_palette_name, "%06d", new_super_palette_id);
    super_palette.name = string("SP") + super_palette_name;
    //------------------------------------------------------------------------
    new_super_palette_id++;
    push_back_palette(super_palette);
    //------------------------------------------------------------------------
    best_match_palette(LaserBoy_palette_set::size() - 1);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
bool LaserBoy_space::call_frame_effect(string name)
{
    int i;
    //------------------------------------------------------------------------
    for(i = 0; i < (int)frame_effect_names.size(); i++)
        if(frame_effect_names[i] == name)
            break;
    //------------------------------------------------------------------------
    if(i < (int)frame_effect_names.size())
        frame_effects[i](&(current_frame()));
    else
        return false;
    return true;
}

//############################################################################
bool LaserBoy_space::call_frame_set_effect(string name)
{
    int i;
    //------------------------------------------------------------------------
    for(i = 0; i < (int)frame_set_effect_names.size(); i++)
        if(frame_set_effect_names[i] == name)
            break;
    //------------------------------------------------------------------------
    if(i < (int)frame_set_effect_names.size())
        frame_set_effects[i](this);
    else
        return false;
    return true;
}

//############################################################################
void LaserBoy_space::load_background_bitmap(struct LaserBoy_bmp* bmp)
{
    bmp_flip(bmp);
    bmp_into_bmp(&background_bmp, bmp);
    show_background_bitmap = true;
    return;
}

//############################################################################
bool LaserBoy_space::load_background_bitmap(string file)
{
    char file_name[81];
    struct LaserBoy_bmp bmp;

    strcpy(file_name, (file).c_str());
    if(bmp_from_file(&bmp, file_name))
    {
        load_background_bitmap(&bmp);
        bmp_free(&bmp);
        bg_file_name = file;
        return true;
    }
    return false;
}

//############################################################################
void LaserBoy_space::clear_background_bitmap()
{
    bmp_uint(&background    , bg_color.as_uint());
    bmp_uint(&background_bmp, bg_color.as_uint());
    bg_file_name = ".";
    return;
}

//############################################################################
void LaserBoy_space::recolor_background()
{
    bmp_uint(&background    , bg_color.as_uint());
    bmp_uint(&background_bmp, bg_color.as_uint());
    if(bg_file_name != ".")
        load_background_bitmap(bg_file_name);
    return;
}

//############################################################################
void LaserBoy_space::load_cfg_file()
{
    ifstream in(LASERBOY_CFG_FILE, ios::in);
    if(in.is_open())
    {
        int     c;
        string  version;
        //--------------------------------------------------------------------
        in >> version;
        if(version == LASERBOY_CFG_VERSION)
        {
            in >> bg_file_name            ;
            in >> c                       ;
            bg_color.r = c                ;
            in >> c                       ;
            bg_color.g = c                ;
            in >> c                       ;
            bg_color.b = c                ;
            in >> sample_rate             ;
            in >> lit_dwell_overhang      ;
            in >> dwell_on_dot            ;
            in >> font_size_factor        ;
            in >> displacement_step       ;
            in >> output_bmp_size         ;
            in >> signal_if_not_z         ;
            in >> unframed_wave_stride    ;
            in >> fulcrum.x               ;
            in >> fulcrum.y               ;
            in >> fulcrum.z               ;
            in >> view_angle.x            ;
            in >> view_angle.y            ;
            in >> view_angle.z            ;
            in >> view_scale.x            ;
            in >> view_scale.y            ;
            in >> view_offset.x           ;
            in >> view_offset.y           ;
            in >> lit_delta_max           ;
            in >> blank_delta_max         ;
            in >> max_dwell_microsec      ;
            in >> insignificant_distance  ;
            in >> insignificant_angle     ;
            in >> recolor_span_factor     ;
            in >> rendered_arc_angle      ;
            in >> rotation_step           ;
            in >> magnitude_step          ;
            magnitude_step_up = 1.00 + magnitude_step;
            magnitude_step_dn = 1.00 - magnitude_step;
            in >> frames_per_second       ;
            in >> still_frame_duration    ;
            in >> show_vertices           ;
            in >> show_blanking           ;
            in >> show_cursors            ;
            in >> show_palettes           ;
            in >> show_stats              ;
            in >> show_fixed_bounds       ;
            in >> show_fixed_origin       ;
            in >> show_floating_axis      ;
            in >> show_floating_bounds    ;
            in >> show_effects_generation ;
            in >> auto_scale_dxf          ;
            in >> maintain_dxf_origin     ;
            in >> loop_animation          ;
            in >> approximate_frame_rate  ;
            in >> black_dwell_vertices    ;
            in >> auto_flatten_z          ;
            in >> no_equivalent_vectors   ;
            in >> no_dots                 ;
            in >> invert_wave_output      ;
            in >> auto_optimize           ;
            in >> no_black_match          ;
            in >> allow_lit_black         ;
            in >> dumb_file_format        ;
            in >> save_txt_rgb            ;
            in >> save_txt_unit           ;
            in >> wave_show_vertices      ;
            in >> wave_show_blanks        ;
            in >> wave_real_time          ;
            in >> wave_show_inverted      ;
            in >> wave_clean_screen       ;
            in >> wave_loop               ;
            in >> wave_offsets[0]         ;
            in >> wave_offsets[1]         ;
            in >> wave_offsets[2]         ;
            in >> wave_offsets[3]         ;
            in >> wave_offsets[4]         ;
            in >> wave_offsets[5]         ;
            in >> wave_offsets[6]         ;
            in >> wave_offsets[7]         ;
            in >> c                       ;
            black_threshold = c           ;
            in >> signal_bit_mask[0]      ;
            in >> signal_bit_mask[1]      ;
            in >> signal_bit_mask[2]      ;
            in >> signal_bit_mask[3]      ;
            in >> signal_bit_mask[4]      ;
            in >> signal_bit_mask[5]      ;
            in >> signal_bit_mask[6]      ;
            in >> signal_bit_mask[7]      ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-03-04-2009")
        {
            in >> bg_file_name            ;
            in >> c                       ;
            bg_color.r = c                ;
            in >> c                       ;
            bg_color.g = c                ;
            in >> c                       ;
            bg_color.b = c                ;
            in >> sample_rate             ;
            in >> lit_dwell_overhang      ;
            in >> dwell_on_dot            ;
            in >> font_size_factor        ;
            in >> displacement_step       ;
            in >> output_bmp_size         ;
            in >> signal_if_not_z         ;
            in >> unframed_wave_stride    ;
            in >> fulcrum.x               ;
            in >> fulcrum.y               ;
            in >> fulcrum.z               ;
            in >> view_angle.x            ;
            in >> view_angle.y            ;
            in >> view_angle.z            ;
            in >> view_scale.x            ;
            in >> view_scale.y            ;
            in >> view_offset.x           ;
            in >> view_offset.y           ;
            in >> lit_delta_max           ;
            in >> blank_delta_max         ;
            in >> max_dwell_microsec      ;
            in >> insignificant_distance  ;
            in >> insignificant_angle     ;
            in >> recolor_span_factor     ;
            in >> rendered_arc_angle      ;
            in >> rotation_step           ;
            in >> magnitude_step          ;
            magnitude_step_up = 1.00 + magnitude_step;
            magnitude_step_dn = 1.00 - magnitude_step;
            in >> frames_per_second       ;
            in >> still_frame_duration    ;
            in >> show_vertices           ;
            in >> show_blanking           ;
            in >> show_cursors            ;
            in >> show_palettes           ;
            in >> show_stats              ;
            in >> show_fixed_bounds       ;
            in >> show_fixed_origin       ;
            in >> show_floating_axis      ;
            in >> show_floating_bounds    ;
            in >> show_effects_generation ;
            in >> auto_scale_dxf          ;
            in >> maintain_dxf_origin     ;
            in >> loop_animation          ;
            in >> approximate_frame_rate  ;
            in >> black_dwell_vertices    ;
            in >> auto_flatten_z          ;
            in >> no_equivalent_vectors   ;
            in >> no_dots                 ;
            in >> invert_wave_output      ;
            in >> auto_optimize           ;
            in >> no_black_match          ;
            in >> allow_lit_black         ;
            in >> dumb_file_format        ;
            in >> wave_show_vertices      ;
            in >> wave_show_blanks        ;
            in >> wave_real_time          ;
            in >> wave_show_inverted      ;
            in >> wave_clean_screen       ;
            in >> wave_loop               ;
            in >> wave_offsets[0]         ;
            in >> wave_offsets[1]         ;
            in >> wave_offsets[2]         ;
            in >> wave_offsets[3]         ;
            in >> wave_offsets[4]         ;
            in >> wave_offsets[5]         ;
            in >> wave_offsets[6]         ;
            in >> wave_offsets[7]         ;
            in >> c                       ;
            black_threshold = c           ;
            in >> signal_bit_mask[0]      ;
            in >> signal_bit_mask[1]      ;
            in >> signal_bit_mask[2]      ;
            in >> signal_bit_mask[3]      ;
            in >> signal_bit_mask[4]      ;
            in >> signal_bit_mask[5]      ;
            in >> signal_bit_mask[6]      ;
            in >> signal_bit_mask[7]      ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-02-10-2009")
        {
            in >> bg_file_name            ;
            in >> c                       ;
            bg_color.r = c                ;
            in >> c                       ;
            bg_color.g = c                ;
            in >> c                       ;
            bg_color.b = c                ;
            in >> sample_rate             ;
            in >> lit_dwell_overhang      ;
            in >> dwell_on_dot            ;
            in >> font_size_factor        ;
            in >> displacement_step       ;
            in >> output_bmp_size         ;
            in >> signal_if_not_z         ;
            in >> fulcrum.x               ;
            in >> fulcrum.y               ;
            in >> fulcrum.z               ;
            in >> view_angle.x            ;
            in >> view_angle.y            ;
            in >> view_angle.z            ;
            in >> view_scale.x            ;
            in >> view_scale.y            ;
            in >> view_offset.x           ;
            in >> view_offset.y           ;
            in >> lit_delta_max           ;
            in >> blank_delta_max         ;
            in >> max_dwell_microsec      ;
            in >> insignificant_distance  ;
            in >> insignificant_angle     ;
            in >> recolor_span_factor     ;
            in >> rendered_arc_angle      ;
            in >> rotation_step           ;
            in >> magnitude_step          ;
            magnitude_step_up = 1.00 + magnitude_step;
            magnitude_step_dn = 1.00 - magnitude_step;
            in >> frames_per_second       ;
            in >> still_frame_duration    ;
            in >> show_vertices           ;
            in >> show_blanking           ;
            in >> show_cursors            ;
            in >> show_palettes           ;
            in >> show_stats              ;
            in >> show_fixed_bounds       ;
            in >> show_fixed_origin       ;
            in >> show_floating_axis      ;
            in >> show_floating_bounds    ;
            in >> show_effects_generation ;
            in >> auto_scale_dxf          ;
            in >> maintain_dxf_origin     ;
            in >> loop_animation          ;
            in >> approximate_frame_rate  ;
            in >> black_dwell_vertices    ;
            in >> auto_flatten_z          ;
            in >> no_equivalent_vectors   ;
            in >> no_dots                 ;
            in >> invert_wave_output      ;
            in >> auto_optimize           ;
            in >> no_black_match          ;
            in >> allow_lit_black         ;
            in >> dumb_file_format        ;
            in >> wave_show_vertices      ;
            in >> wave_show_blanks        ;
            in >> wave_real_time          ;
            in >> wave_show_inverted      ;
            in >> wave_clean_screen       ;
            in >> wave_loop               ;
            in >> wave_offsets[0]         ;
            in >> wave_offsets[1]         ;
            in >> wave_offsets[2]         ;
            in >> wave_offsets[3]         ;
            in >> wave_offsets[4]         ;
            in >> wave_offsets[5]         ;
            in >> wave_offsets[6]         ;
            in >> wave_offsets[7]         ;
            in >> c                       ;
            black_threshold = c           ;
            in >> signal_bit_mask[0]      ;
            in >> signal_bit_mask[1]      ;
            in >> signal_bit_mask[2]      ;
            in >> signal_bit_mask[3]      ;
            in >> signal_bit_mask[4]      ;
            in >> signal_bit_mask[5]      ;
            in >> signal_bit_mask[6]      ;
            in >> signal_bit_mask[7]      ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-01-16-2009")
        {
            in >> bg_file_name            ;
            in >> c                       ;
            bg_color.r = c                ;
            in >> c                       ;
            bg_color.g = c                ;
            in >> c                       ;
            bg_color.b = c                ;
            in >> sample_rate             ;
            in >> lit_dwell_overhang      ;
            in >> dwell_on_dot            ;
            in >> font_size_factor        ;
            in >> displacement_step       ;
            in >> output_bmp_size         ;
            in >> fulcrum.x               ;
            in >> fulcrum.y               ;
            in >> fulcrum.z               ;
            in >> view_angle.x            ;
            in >> view_angle.y            ;
            in >> view_angle.z            ;
            in >> view_scale.x            ;
            in >> view_scale.y            ;
            in >> view_offset.x           ;
            in >> view_offset.y           ;
            in >> lit_delta_max           ;
            in >> blank_delta_max         ;
            in >> max_dwell_microsec      ;
            in >> insignificant_distance  ;
            in >> insignificant_angle     ;
            in >> recolor_span_factor     ;
            in >> rendered_arc_angle      ;
            in >> rotation_step           ;
            in >> magnitude_step          ;
            magnitude_step_up = 1.00 + magnitude_step;
            magnitude_step_dn = 1.00 - magnitude_step;
            in >> frames_per_second       ;
            in >> still_frame_duration    ;
            in >> show_vertices           ;
            in >> show_blanking           ;
            in >> show_cursors            ;
            in >> show_palettes           ;
            in >> show_stats              ;
            in >> show_fixed_bounds       ;
            in >> show_fixed_origin       ;
            in >> show_floating_axis      ;
            in >> show_floating_bounds    ;
            in >> show_effects_generation ;
            in >> auto_scale_dxf          ;
            in >> maintain_dxf_origin     ;
            in >> loop_animation          ;
            in >> approximate_frame_rate  ;
            in >> black_dwell_vertices    ;
            in >> auto_flatten_z          ;
            in >> no_equivalent_vectors   ;
            in >> no_dots                 ;
            in >> invert_wave_output      ;
            in >> auto_optimize           ;
            in >> no_black_match          ;
            in >> allow_lit_black         ;
            in >> dumb_file_format        ;
            in >> wave_show_vertices      ;
            in >> wave_show_blanks        ;
            in >> wave_real_time          ;
            in >> wave_show_inverted      ;
            in >> wave_clean_screen       ;
            in >> wave_loop               ;
            in >> wave_offsets[0]         ;
            in >> wave_offsets[1]         ;
            in >> wave_offsets[2]         ;
            in >> wave_offsets[3]         ;
            in >> wave_offsets[4]         ;
            in >> wave_offsets[5]         ;
            in >> c                       ;
            black_threshold = c           ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-01-09-2009")
        {
            in >> bg_file_name            ;
            in >> c                       ;
            bg_color.r = c                ;
            in >> c                       ;
            bg_color.g = c                ;
            in >> c                       ;
            bg_color.b = c                ;
            in >> sample_rate             ;
            in >> lit_dwell_overhang      ;
            in >> font_size_factor        ;
            in >> displacement_step       ;
            in >> output_bmp_size         ;
            in >> fulcrum.x               ;
            in >> fulcrum.y               ;
            in >> fulcrum.z               ;
            in >> view_angle.x            ;
            in >> view_angle.y            ;
            in >> view_angle.z            ;
            in >> view_scale.x            ;
            in >> view_scale.y            ;
            in >> view_offset.x           ;
            in >> view_offset.y           ;
            in >> lit_delta_max           ;
            in >> blank_delta_max         ;
            in >> max_dwell_microsec      ;
            in >> insignificant_distance  ;
            in >> insignificant_angle     ;
            in >> recolor_span_factor     ;
            in >> rendered_arc_angle      ;
            in >> rotation_step           ;
            in >> frames_per_second       ;
            in >> still_frame_duration    ;
            in >> show_vertices           ;
            in >> show_blanking           ;
            in >> show_cursors            ;
            in >> show_palettes           ;
            in >> show_stats              ;
            in >> show_fixed_bounds       ;
            in >> show_fixed_origin       ;
            in >> show_floating_axis      ;
            in >> show_floating_bounds    ;
            in >> show_effects_generation ;
            in >> auto_scale_dxf          ;
            in >> maintain_dxf_origin     ;
            in >> loop_animation          ;
            in >> approximate_frame_rate  ;
            in >> black_dwell_vertices    ;
            in >> auto_flatten_z          ;
            in >> no_equivalent_vectors   ;
            in >> no_dots                 ;
            in >> invert_wave_output      ;
            in >> auto_optimize           ;
            in >> no_black_match          ;
            in >> allow_lit_black         ;
            in >> dumb_file_format        ;
            in >> wave_show_vertices      ;
            in >> wave_show_blanks        ;
            in >> wave_real_time          ;
            in >> wave_show_inverted      ;
            in >> wave_clean_screen       ;
            in >> wave_loop               ;
            in >> wave_offsets[0]         ;
            in >> wave_offsets[1]         ;
            in >> wave_offsets[2]         ;
            in >> wave_offsets[3]         ;
            in >> wave_offsets[4]         ;
            in >> wave_offsets[5]         ;
            in >> c                       ;
            black_threshold = c           ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-01-06-2009")
        {
            in >> bg_file_name            ;
            in >> c                       ;
            bg_color.r = c                ;
            in >> c                       ;
            bg_color.g = c                ;
            in >> c                       ;
            bg_color.b = c                ;
            in >> sample_rate             ;
            in >> lit_dwell_overhang      ;
            in >> font_size_factor        ;
            in >> displacement_step       ;
            in >> output_bmp_size         ;
            in >> fulcrum.x               ;
            in >> fulcrum.y               ;
            in >> fulcrum.z               ;
            in >> view_angle.x            ;
            in >> view_angle.y            ;
            in >> view_angle.z            ;
            in >> view_scale.x            ;
            in >> view_scale.y            ;
            in >> view_offset.x           ;
            in >> view_offset.y           ;
            in >> lit_delta_max           ;
            in >> blank_delta_max         ;
            in >> max_dwell_microsec      ;
            in >> insignificant_distance  ;
            in >> insignificant_angle     ;
            in >> recolor_span_factor     ;
            in >> rendered_arc_angle      ;
            in >> rotation_step           ;
            in >> frames_per_second       ;
            in >> still_frame_duration    ;
            in >> show_vertices           ;
            in >> show_blanking           ;
            in >> show_cursors            ;
            in >> show_palettes           ;
            in >> show_stats              ;
            in >> show_fixed_bounds       ;
            in >> show_fixed_origin       ;
            in >> show_floating_axis      ;
            in >> show_floating_bounds    ;
            in >> show_effects_generation ;
            in >> auto_scale_dxf          ;
            in >> maintain_dxf_origin     ;
            in >> loop_animation          ;
            in >> approximate_frame_rate  ;
            in >> black_dwell_vertices    ;
            in >> auto_flatten_z          ;
            in >> no_equivalent_vectors   ;
            in >> invert_wave_output      ;
            in >> auto_optimize           ;
            in >> no_black_match          ;
            in >> allow_lit_black         ;
            in >> dumb_file_format        ;
            in >> wave_show_vertices      ;
            in >> wave_show_blanks        ;
            in >> wave_real_time          ;
            in >> wave_show_inverted      ;
            in >> wave_clean_screen       ;
            in >> wave_loop               ;
            in >> wave_offsets[0]         ;
            in >> wave_offsets[1]         ;
            in >> wave_offsets[2]         ;
            in >> wave_offsets[3]         ;
            in >> wave_offsets[4]         ;
            in >> wave_offsets[5]         ;
            in >> c                       ;
            black_threshold = c           ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-11-11-2008")
        {
            in >> bg_file_name            ;
            in >> c                       ;
            bg_color.r = c                ;
            in >> c                       ;
            bg_color.g = c                ;
            in >> c                       ;
            bg_color.b = c                ;
            in >> sample_rate             ;
            in >> lit_dwell_overhang      ;
            in >> font_size_factor        ;
            in >> displacement_step       ;
            in >> output_bmp_size         ;
            in >> fulcrum.x               ;
            in >> fulcrum.y               ;
            in >> fulcrum.z               ;
            in >> view_angle.x            ;
            in >> view_angle.y            ;
            in >> view_angle.z            ;
            in >> view_scale.x            ;
            in >> view_scale.y            ;
            in >> view_offset.x           ;
            in >> view_offset.y           ;
            in >> lit_delta_max           ;
            in >> blank_delta_max         ;
            in >> max_dwell_microsec      ;
            in >> insignificant_distance  ;
            in >> insignificant_angle     ;
            in >> recolor_span_factor     ;
            in >> rendered_arc_angle      ;
            in >> rotation_step           ;
            in >> frames_per_second       ;
            in >> still_frame_duration    ;
            in >> show_vertices           ;
            in >> show_blanking           ;
            in >> show_cursors            ;
            in >> show_palettes           ;
            in >> show_stats              ;
            in >> show_fixed_bounds       ;
            in >> show_fixed_origin       ;
            in >> show_floating_axis      ;
            in >> show_floating_bounds    ;
            in >> show_effects_generation ;
            in >> auto_scale_dxf          ;
            in >> maintain_dxf_origin     ;
            in >> loop_animation          ;
            in >> approximate_frame_rate  ;
            in >> black_dwell_vertices    ;
            in >> auto_flatten_z          ;
            in >> no_equivalent_vectors   ;
            in >> invert_wave_output      ;
            in >> auto_optimize           ;
            in >> no_black_match          ;
            in >> dumb_file_format        ;
            in >> wave_show_vertices      ;
            in >> wave_show_blanks        ;
            in >> wave_real_time          ;
            in >> wave_show_inverted      ;
            in >> wave_clean_screen       ;
            in >> wave_loop               ;
            in >> wave_offsets[0]         ;
            in >> wave_offsets[1]         ;
            in >> wave_offsets[2]         ;
            in >> wave_offsets[3]         ;
            in >> wave_offsets[4]         ;
            in >> wave_offsets[5]         ;
            in >> c                       ;
            black_threshold = c           ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-10-10-2008")
        {
            in >> bg_file_name           ;
            in >> c                      ;
            bg_color.r = c               ;
            in >> c                      ;
            bg_color.g = c               ;
            in >> c                      ;
            bg_color.b = c               ;
            in >> sample_rate            ;
            in >> lit_dwell_overhang     ;
            in >> font_size_factor       ;
            in >> displacement_step      ;
            in >> output_bmp_size        ;
            in >> fulcrum.x              ;
            in >> fulcrum.y              ;
            in >> fulcrum.z              ;
            in >> view_angle.x           ;
            in >> view_angle.y           ;
            in >> view_angle.z           ;
            in >> view_scale.x           ;
            in >> view_scale.y           ;
            in >> view_offset.x          ;
            in >> view_offset.y          ;
            in >> lit_delta_max          ;
            blank_delta_max = lit_delta_max;
            in >> max_dwell_microsec     ;
            in >> insignificant_distance ;
            in >> insignificant_angle    ;
            in >> recolor_span_factor    ;
            in >> rendered_arc_angle     ;
            in >> rotation_step          ;
            in >> frames_per_second      ;
            in >> still_frame_duration   ;
            in >> show_vertices          ;
            in >> show_blanking          ;
            in >> show_cursors           ;
            in >> show_palettes          ;
            in >> show_stats             ;
            in >> show_fixed_bounds      ;
            in >> show_fixed_origin      ;
            in >> show_floating_axis     ;
            in >> show_floating_bounds   ;
            in >> show_effects_generation;
            in >> auto_scale_dxf         ;
            in >> maintain_dxf_origin    ;
            in >> loop_animation         ;
            in >> approximate_frame_rate ;
            in >> black_dwell_vertices   ;
            in >> auto_flatten_z         ;
            in >> no_equivalent_vectors  ;
            in >> invert_wave_output     ;
            in >> auto_optimize          ;
            in >> dumb_file_format       ;
            in >> wave_show_vertices     ;
            in >> wave_show_blanks       ;
            in >> wave_real_time         ;
            in >> wave_show_inverted     ;
            in >> wave_clean_screen      ;
            in >> wave_loop              ;
            in >> wave_offsets[0]        ;
            in >> wave_offsets[1]        ;
            in >> wave_offsets[2]        ;
            in >> wave_offsets[3]        ;
            in >> wave_offsets[4]        ;
            in >> wave_offsets[5]        ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-09-03-2008")
        {
            in >> bg_file_name           ;
            in >> c                      ;
            bg_color.r = c               ;
            in >> c                      ;
            bg_color.g = c               ;
            in >> c                      ;
            bg_color.b = c               ;
            in >> sample_rate            ;
            in >> lit_dwell_overhang     ;
            in >> font_size_factor       ;
            in >> displacement_step      ;
            in >> output_bmp_size        ;
            in >> fulcrum.x              ;
            in >> fulcrum.y              ;
            in >> fulcrum.z              ;
            in >> view_angle.x           ;
            in >> view_angle.y           ;
            in >> view_angle.z           ;
            in >> view_scale.x           ;
            in >> view_scale.y           ;
            in >> view_offset.x          ;
            in >> view_offset.y          ;
            in >> lit_delta_max          ;
            blank_delta_max = lit_delta_max;
            in >> max_dwell_microsec     ;
            in >> insignificant_distance ;
            in >> insignificant_angle    ;
            in >> recolor_span_factor    ;
            in >> rendered_arc_angle     ;
            in >> rotation_step          ;
            in >> frames_per_second      ;
            in >> still_frame_duration   ;
            in >> show_vertices          ;
            in >> show_blanking          ;
            in >> show_cursors           ;
            in >> show_palettes          ;
            in >> show_stats             ;
            in >> show_fixed_bounds      ;
            in >> show_fixed_origin      ;
            in >> show_floating_axis     ;
            in >> show_floating_bounds   ;
            in >> show_effects_generation;
            in >> auto_scale_dxf         ;
            in >> maintain_dxf_origin    ;
            in >> loop_animation         ;
            in >> approximate_frame_rate ;
            in >> black_dwell_vertices   ;
            in >> auto_flatten_z         ;
            in >> no_equivalent_vectors  ;
            in >> invert_wave_output     ;
            in >> auto_optimize          ;
            in >> dumb_file_format       ;
            in >> wave_show_vertices     ;
            in >> wave_show_blanks       ;
            in >> wave_real_time         ;
            in >> wave_show_inverted     ;
            in >> wave_clean_screen      ;
            in >> wave_offsets[0]        ;
            in >> wave_offsets[1]        ;
            in >> wave_offsets[2]        ;
            in >> wave_offsets[3]        ;
            in >> wave_offsets[4]        ;
            in >> wave_offsets[5]        ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-08-28-2008")
        {
            in >> bg_file_name           ;
            in >> c                      ;
            bg_color.r = c               ;
            in >> c                      ;
            bg_color.g = c               ;
            in >> c                      ;
            bg_color.b = c               ;
            in >> sample_rate            ;
            in >> lit_dwell_overhang     ;
            in >> font_size_factor       ;
            in >> displacement_step      ;
            in >> output_bmp_size        ;
            in >> fulcrum.x              ;
            in >> fulcrum.y              ;
            in >> fulcrum.z              ;
            in >> view_angle.x           ;
            in >> view_angle.y           ;
            in >> view_angle.z           ;
            in >> view_scale.x           ;
            in >> view_scale.y           ;
            in >> view_offset.x          ;
            in >> view_offset.y          ;
            in >> lit_delta_max          ;
            blank_delta_max = lit_delta_max;
            in >> max_dwell_microsec     ;
            in >> insignificant_distance ;
            in >> insignificant_angle    ;
            in >> recolor_span_factor    ;
            in >> rendered_arc_angle     ;
            in >> rotation_step          ;
            in >> frames_per_second      ;
            in >> still_frame_duration   ;
            in >> show_vertices          ;
            in >> show_blanking          ;
            in >> show_cursors           ;
            in >> show_palettes          ;
            in >> show_stats             ;
            in >> show_fixed_bounds      ;
            in >> show_fixed_origin      ;
            in >> show_floating_axis     ;
            in >> show_floating_bounds   ;
            in >> show_effects_generation;
            in >> auto_scale_dxf         ;
            in >> maintain_dxf_origin    ;
            in >> loop_animation         ;
            in >> approximate_frame_rate ;
            in >> black_dwell_vertices   ;
            in >> auto_flatten_z         ;
            in >> no_equivalent_vectors  ;
            in >> invert_wave_output     ;
            in >> auto_optimize          ;
            in >> wave_show_vertices     ;
            in >> wave_show_blanks       ;
            in >> wave_real_time         ;
            in >> wave_show_inverted     ;
            in >> wave_clean_screen      ;
            in >> wave_offsets[0]        ;
            in >> wave_offsets[1]        ;
            in >> wave_offsets[2]        ;
            in >> wave_offsets[3]        ;
            in >> wave_offsets[4]        ;
            in >> wave_offsets[5]        ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-07-24-2008")
        {
            in >> bg_file_name           ;
            in >> c                      ;
            bg_color.r = c               ;
            in >> c                      ;
            bg_color.g = c               ;
            in >> c                      ;
            bg_color.b = c               ;
            in >> sample_rate            ;
            in >> lit_dwell_overhang     ;
            in >> font_size_factor       ;
            in >> displacement_step      ;
            in >> output_bmp_size        ;
            in >> fulcrum.x              ;
            in >> fulcrum.y              ;
            in >> fulcrum.z              ;
            in >> view_angle.x           ;
            in >> view_angle.y           ;
            in >> view_angle.z           ;
            in >> view_scale.x           ;
            in >> view_scale.y           ;
            in >> view_scale.z           ;
            in >> view_offset.x          ;
            in >> view_offset.y          ;
            in >> view_offset.z          ;
            in >> lit_delta_max          ;
            blank_delta_max = lit_delta_max;
            in >> max_dwell_microsec     ;
            in >> insignificant_distance ;
            in >> insignificant_angle    ;
            in >> recolor_span_factor    ;
            in >> rendered_arc_angle     ;
            in >> rotation_step          ;
            in >> frames_per_second      ;
            in >> still_frame_duration   ;
            in >> show_vertices          ;
            in >> show_blanking          ;
            in >> show_cursors           ;
            in >> show_palettes          ;
            in >> show_stats             ;
            in >> show_fixed_bounds      ;
            in >> show_fixed_origin      ;
            in >> show_floating_axis     ;
            in >> show_floating_bounds   ;
            in >> show_effects_generation;
            in >> auto_scale_dxf         ;
            in >> maintain_dxf_origin    ;
            in >> loop_animation         ;
            in >> approximate_frame_rate ;
            in >> black_dwell_vertices   ;
            in >> auto_flatten_z         ;
            in >> no_equivalent_vectors  ;
            in >> invert_wave_output     ;
            in >> auto_optimize          ;
            in >> wave_show_vertices     ;
            in >> wave_show_blanks       ;
            in >> wave_real_time         ;
            in >> wave_show_inverted     ;
            in >> wave_clean_screen      ;
            in >> wave_offsets[0]        ;
            in >> wave_offsets[1]        ;
            in >> wave_offsets[2]        ;
            in >> wave_offsets[3]        ;
            in >> wave_offsets[4]        ;
            in >> wave_offsets[5]        ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-07-17-2008")
        {
            in >> bg_file_name           ;
            in >> c                      ;
            bg_color.r = c               ;
            in >> c                      ;
            bg_color.g = c               ;
            in >> c                      ;
            bg_color.b = c               ;
            in >> sample_rate            ;
            in >> lit_dwell_overhang     ;
            in >> font_size_factor       ;
            in >> displacement_step      ;
            in >> output_bmp_size        ;
            in >> fulcrum.x              ;
            in >> fulcrum.y              ;
            in >> fulcrum.z              ;
            in >> view_angle.x           ;
            in >> view_angle.y           ;
            in >> view_angle.z           ;
            in >> view_scale.x           ;
            in >> view_scale.y           ;
            in >> view_scale.z           ;
            in >> view_offset.x          ;
            in >> view_offset.y          ;
            in >> view_offset.z          ;
            in >> lit_delta_max          ;
            blank_delta_max = lit_delta_max;
            in >> max_dwell_microsec     ;
            in >> insignificant_distance ;
            in >> insignificant_angle    ;
            in >> recolor_span_factor    ;
            in >> rendered_arc_angle     ;
            in >> frames_per_second      ;
            in >> still_frame_duration   ;
            in >> show_vertices          ;
            in >> show_blanking          ;
            in >> show_cursors           ;
            in >> show_palettes          ;
            in >> show_stats             ;
            in >> show_fixed_bounds      ;
            in >> show_fixed_origin      ;
            in >> show_floating_axis     ;
            in >> show_floating_bounds   ;
            in >> show_effects_generation;
            in >> auto_scale_dxf         ;
            in >> maintain_dxf_origin    ;
            in >> loop_animation         ;
            in >> approximate_frame_rate ;
            in >> black_dwell_vertices   ;
            in >> auto_flatten_z         ;
            in >> no_equivalent_vectors  ;
            in >> invert_wave_output     ;
            in >> auto_optimize          ;
            in >> wave_show_vertices     ;
            in >> wave_show_blanks       ;
            in >> wave_real_time         ;
            in >> wave_show_inverted     ;
            in >> wave_clean_screen      ;
            in >> wave_offsets[0]        ;
            in >> wave_offsets[1]        ;
            in >> wave_offsets[2]        ;
            in >> wave_offsets[3]        ;
            in >> wave_offsets[4]        ;
            in >> wave_offsets[5]        ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-07-15-2008")
        {
            in >> bg_file_name           ;
            in >> c                      ;
            bg_color.r = c               ;
            in >> c                      ;
            bg_color.g = c               ;
            in >> c                      ;
            bg_color.b = c               ;
            in >> sample_rate            ;
            in >> lit_dwell_overhang     ;
            in >> font_size_factor       ;
            in >> displacement_step      ;
            in >> output_bmp_size        ;
            in >> fulcrum.x              ;
            in >> fulcrum.y              ;
            in >> fulcrum.z              ;
            in >> view_angle.x           ;
            in >> view_angle.y           ;
            in >> view_angle.z           ;
            in >> view_scale.x           ;
            in >> view_scale.y           ;
            in >> view_scale.z           ;
            in >> view_offset.x          ;
            in >> view_offset.y          ;
            in >> view_offset.z          ;
            in >> lit_delta_max          ;
            blank_delta_max = lit_delta_max;
            in >> max_dwell_microsec     ;
            in >> insignificant_distance ;
            in >> insignificant_angle    ;
            in >> recolor_span_factor    ;
            in >> rendered_arc_angle     ;
            in >> frames_per_second      ;
            in >> still_frame_duration   ;
            in >> show_vertices          ;
            in >> show_blanking          ;
            in >> show_cursors           ;
            in >> show_palettes          ;
            in >> show_stats             ;
            in >> show_fixed_bounds      ;
            in >> show_fixed_origin      ;
            in >> show_floating_axis     ;
            in >> show_floating_bounds   ;
            in >> show_effects_generation;
            in >> auto_scale_dxf         ;
            in >> maintain_dxf_origin    ;
            in >> loop_animation         ;
            in >> approximate_frame_rate ;
            in >> black_dwell_vertices   ;
            in >> auto_flatten_z         ;
            in >> no_equivalent_vectors  ;
            in >> auto_optimize          ;
            in >> wave_show_vertices     ;
            in >> wave_show_blanks       ;
            in >> wave_real_time         ;
            in >> wave_show_inverted     ;
            in >> wave_clean_screen      ;
            in >> wave_offsets[0]        ;
            in >> wave_offsets[1]        ;
            in >> wave_offsets[2]        ;
            in >> wave_offsets[3]        ;
            in >> wave_offsets[4]        ;
            in >> wave_offsets[5]        ;
        }
        //--------------------------------------------------------------------
        else if(version == "LaserBoy.wtf-06-03-2008")
        {
            in >> bg_file_name           ;
            in >> c                      ;
            bg_color.r = c               ;
            in >> c                      ;
            bg_color.g = c               ;
            in >> c                      ;
            bg_color.b = c               ;
            in >> sample_rate            ;
            in >> lit_dwell_overhang     ;
            in >> font_size_factor       ;
            in >> displacement_step      ;
            in >> output_bmp_size        ;
            in >> fulcrum.x              ;
            in >> fulcrum.y              ;
            in >> fulcrum.z              ;
            in >> view_angle.x           ;
            in >> view_angle.y           ;
            in >> view_angle.z           ;
            in >> view_scale.x           ;
            in >> view_scale.y           ;
            in >> view_scale.z           ;
            in >> view_offset.x          ;
            in >> view_offset.y          ;
            in >> view_offset.z          ;
            in >> lit_delta_max          ;
            blank_delta_max = lit_delta_max;
            in >> max_dwell_microsec     ;
            in >> insignificant_distance ;
            in >> insignificant_angle    ;
            in >> recolor_span_factor    ;
            in >> rendered_arc_angle     ;
            in >> frames_per_second      ;
            in >> still_frame_duration   ;
            in >> show_vertices          ;
            in >> show_blanking          ;
            in >> show_cursors           ;
            in >> show_palettes          ;
            in >> show_stats             ;
            in >> show_fixed_bounds      ;
            in >> show_fixed_origin      ;
            in >> show_floating_axis     ;
            in >> show_floating_bounds   ;
            in >> show_effects_generation;
            in >> auto_scale_dxf         ;
            in >> maintain_dxf_origin    ;
            in >> loop_animation         ;
            in >> approximate_frame_rate ;
            in >> black_dwell_vertices   ;
            in >> auto_flatten_z         ;
            in >> auto_optimize          ;
            in >> wave_show_vertices     ;
            in >> wave_show_blanks       ;
            in >> wave_real_time         ;
            in >> wave_show_inverted     ;
            in >> wave_clean_screen      ;
            in >> wave_offsets[0]        ;
            in >> wave_offsets[1]        ;
            in >> wave_offsets[2]        ;
            in >> wave_offsets[3]        ;
            in >> wave_offsets[4]        ;
            in >> wave_offsets[5]        ;
        }
        in.close();
        //--------------------------------------------------------------------
        if(version != LASERBOY_CFG_VERSION)
            save_cfg_file();
    }
    return;
}

//############################################################################
void LaserBoy_space::save_cfg_file()
{
    ofstream out(LASERBOY_CFG_FILE, ios::out);
    if(out.is_open())
    {
        out << LASERBOY_CFG_VERSION     << endl;
        out << bg_file_name.c_str()     << endl;
        out << (int)(bg_color.r)        << endl;
        out << (int)(bg_color.g)        << endl;
        out << (int)(bg_color.b)        << endl;
        out << sample_rate              << endl;
        out << lit_dwell_overhang       << endl;
        out << dwell_on_dot             << endl;
        out << font_size_factor         << endl;
        out << displacement_step        << endl;
        out << output_bmp_size          << endl;
        out << signal_if_not_z          << endl;
        out << unframed_wave_stride     << endl;
        out << fulcrum.x                << endl;
        out << fulcrum.y                << endl;
        out << fulcrum.z                << endl;
        out << view_angle.x             << endl;
        out << view_angle.y             << endl;
        out << view_angle.z             << endl;
        out << view_scale.x             << endl;
        out << view_scale.y             << endl;
        out << view_offset.x            << endl;
        out << view_offset.y            << endl;
        out << lit_delta_max            << endl;
        out << blank_delta_max          << endl;
        out << max_dwell_microsec       << endl;
        out << insignificant_distance   << endl;
        out << insignificant_angle      << endl;
        out << recolor_span_factor      << endl;
        out << rendered_arc_angle       << endl;
        out << rotation_step            << endl;
        out << magnitude_step           << endl;
        out << frames_per_second        << endl;
        out << still_frame_duration     << endl;
        out << show_vertices            << endl;
        out << show_blanking            << endl;
        out << show_cursors             << endl;
        out << show_palettes            << endl;
        out << show_stats               << endl;
        out << show_fixed_bounds        << endl;
        out << show_fixed_origin        << endl;
        out << show_floating_axis       << endl;
        out << show_floating_bounds     << endl;
        out << show_effects_generation  << endl;
        out << auto_scale_dxf           << endl;
        out << maintain_dxf_origin      << endl;
        out << loop_animation           << endl;
        out << approximate_frame_rate   << endl;
        out << black_dwell_vertices     << endl;
        out << auto_flatten_z           << endl;
        out << no_equivalent_vectors    << endl;
        out << no_dots                  << endl;
        out << invert_wave_output       << endl;
        out << auto_optimize            << endl;
        out << no_black_match           << endl;
        out << allow_lit_black          << endl;
        out << dumb_file_format         << endl;
        out << save_txt_rgb             << endl;
        out << save_txt_unit            << endl;
        out << wave_show_vertices       << endl;
        out << wave_show_blanks         << endl;
        out << wave_real_time           << endl;
        out << wave_show_inverted       << endl;
        out << wave_clean_screen        << endl;
        out << wave_loop                << endl;
        out << wave_offsets[0]          << endl;
        out << wave_offsets[1]          << endl;
        out << wave_offsets[2]          << endl;
        out << wave_offsets[3]          << endl;
        out << wave_offsets[4]          << endl;
        out << wave_offsets[5]          << endl;
        out << wave_offsets[6]          << endl;
        out << wave_offsets[7]          << endl;
        out << (int)(black_threshold)   << endl;
        out << signal_bit_mask[0]       << endl;
        out << signal_bit_mask[1]       << endl;
        out << signal_bit_mask[2]       << endl;
        out << signal_bit_mask[3]       << endl;
        out << signal_bit_mask[4]       << endl;
        out << signal_bit_mask[5]       << endl;
        out << signal_bit_mask[6]       << endl;
        out << signal_bit_mask[7]       << endl << endl;
        out << "If this file is currupted, delete it!" << endl;
        out.close();
    }
    return;
}

//############################################################################
bool LaserBoy_space::apply_wave_offsets_prep(string file)
{
    ifstream wave_ifstream;
    ofstream wave_ofstream;

    u_int    bmp_bg_color = bmp.make_rgb(bg_color.r, bg_color.g, bg_color.b);
    //------------------------------------------------------------------------
    if(rename(file.c_str(), (file + ".tmp").c_str()))
        return false;
    //------------------------------------------------------------------------
    wave_ifstream.open((file + ".tmp").c_str(), ios::in | ios::binary);
    if(!wave_ifstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header(wave_ifstream);
    if(    wave_in_header.version == "!LaserBoy!"
        || wave_in_header.num_channels < 6
      )
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    LaserBoy_wave_sample wave_sample(wave_in_header.num_channels);
    //------------------------------------------------------------------------
    wave_ofstream.open(file.c_str(), ios::out | ios::binary);
    if(!wave_ofstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    apply_wave_offsets(wave_ifstream, wave_ofstream, wave_in_header);
    //------------------------------------------------------------------------
    wave_ifstream.close();
    wave_ofstream.close();
    remove((file + ".tmp").c_str());
    return true;
}

//############################################################################
bool LaserBoy_space::overwirte_wave_offsets_prep(string file)
{
    ifstream  wave_ifstream;
    ofstream  wave_ofstream;
    //------------------------------------------------------------------------
    u_int bmp_bg_color = bmp.make_rgb(bg_color.r, bg_color.g, bg_color.b);
    //------------------------------------------------------------------------
    if(rename(   file.c_str()
               , (file + ".tmp").c_str()
             )
      )
        return false;
    //------------------------------------------------------------------------
    wave_ifstream.open((file + ".tmp").c_str(), ios::in | ios::binary);
    if(!wave_ifstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header(wave_ifstream);
    if(    wave_in_header.version == "!LaserBoy!"
        || wave_in_header.num_channels < 6
      )
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    LaserBoy_wave_sample wave_sample(wave_in_header.num_channels);
    //------------------------------------------------------------------------
    wave_ofstream.open(file.c_str(), ios::out | ios::binary);
    if(!wave_ofstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    overwirte_wave_offsets(wave_ifstream, wave_ofstream, wave_in_header);
    //------------------------------------------------------------------------
    wave_ifstream.close();
    wave_ofstream.close();
    remove((file + ".tmp").c_str());
    return true;
}

//############################################################################
bool LaserBoy_space::invert_wave_prep(string file)
{
    ifstream  wave_ifstream;
    ofstream  wave_ofstream;
    //------------------------------------------------------------------------
    u_int bmp_bg_color = bmp.make_rgb(bg_color.r, bg_color.g, bg_color.b);
    //------------------------------------------------------------------------
    if(rename(   file.c_str()
               , (file + ".tmp").c_str()
             )
      )
        return false;
    //------------------------------------------------------------------------
    wave_ifstream.open((file + ".tmp").c_str(), ios::in | ios::binary);
    if(!wave_ifstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header(wave_ifstream);
    if(wave_in_header.version == "!LaserBoy!")
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    wave_ofstream.open(file.c_str(), ios::out | ios::binary);
    if(!wave_ofstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    invert_wave(wave_ifstream, wave_ofstream, wave_in_header);
    //------------------------------------------------------------------------
    wave_ifstream.close();
    wave_ofstream.close();
    remove((file + ".tmp").c_str());
    return true;
}

//############################################################################
bool LaserBoy_space::invert_signals_prep(string file)
{
    ifstream  wave_ifstream;
    ofstream  wave_ofstream;
    //------------------------------------------------------------------------
    u_int bmp_bg_color = bmp.make_rgb(bg_color.r, bg_color.g, bg_color.b);
    //------------------------------------------------------------------------
    if(rename(   file.c_str()
               , (file + ".tmp").c_str()
             )
      )
        return false;
    //------------------------------------------------------------------------
    wave_ifstream.open((file + ".tmp").c_str(), ios::in | ios::binary);
    if(!wave_ifstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header(wave_ifstream);
    if(wave_in_header.version == "!LaserBoy!")
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    wave_ofstream.open(file.c_str(), ios::out | ios::binary);
    if(!wave_ofstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    invert_signals(wave_ifstream, wave_ofstream, wave_in_header);
    //------------------------------------------------------------------------
    wave_ifstream.close();
    wave_ofstream.close();
    remove((file + ".tmp").c_str());
    return true;
}

//############################################################################
bool LaserBoy_space::clear_wave_polarity_list_prep(string file)
{
    ifstream  wave_ifstream;
    ofstream  wave_ofstream;
    //------------------------------------------------------------------------
    u_int bmp_bg_color = bmp.make_rgb(bg_color.r, bg_color.g, bg_color.b);
    //------------------------------------------------------------------------
    if(rename(   file.c_str()
               , (file + ".tmp").c_str()
             )
      )
        return false;
    //------------------------------------------------------------------------
    wave_ifstream.open((file + ".tmp").c_str(), ios::in | ios::binary);
    if(!wave_ifstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header(wave_ifstream);
    //------------------------------------------------------------------------
    if(wave_in_header.version == "!LaserBoy!")
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    wave_ofstream.open(file.c_str(), ios::out | ios::binary);
    if(!wave_ofstream.is_open())
    {
        undo_wave_temp(wave_ifstream, wave_ofstream, file);
        return false;
    }
    //------------------------------------------------------------------------
    clear_wave_polarity_list(wave_ifstream, wave_ofstream, wave_in_header);
    //------------------------------------------------------------------------
    wave_ifstream.close();
    wave_ofstream.close();
    remove((file + ".tmp").c_str());
    return true;
}

//############################################################################
bool LaserBoy_space::split_wave_XY_r_g_b_i_LR_prep(   string file
                                                    , string dir_in
                                                    , string dir_laser
                                                    , string dir_audio
                                                  )
{
    ifstream        wave_ifstream   ;
    ofstream        wave_ofstream_XY,
                    wave_ofstream_r ,
                    wave_ofstream_g ,
                    wave_ofstream_b ,
                    wave_ofstream_i ,
                    wave_ofstream_LR;
    //------------------------------------------------------------------------
    u_int bmp_bg_color = bmp.make_rgb(bg_color.r, bg_color.g, bg_color.b);
    //------------------------------------------------------------------------
    wave_ifstream.open((dir_in + file).c_str(), ios::in | ios::binary);
    if(!wave_ifstream.is_open())
        return false;
    //------------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header(wave_ifstream);
    if(    wave_in_header.version == "!LaserBoy!"
        || wave_in_header.num_channels < 6
      )
    {
        wave_ifstream.close();
        return false;
    }
    //------------------------------------------------------------------------
    LaserBoy_wave_sample wave_sample(wave_in_header.num_channels);
    //------------------------------------------------------------------------
    file = file.substr(0, file.size() - 4); // remove .wav
    //------------------------------------------------------------------------
    wave_ofstream_XY.open((dir_laser + file + "_XY.wav").c_str(), ios::out | ios::binary);
    wave_ofstream_r .open((dir_laser + file + "_r.wav" ).c_str(), ios::out | ios::binary);
    wave_ofstream_g .open((dir_laser + file + "_g.wav" ).c_str(), ios::out | ios::binary);
    wave_ofstream_b .open((dir_laser + file + "_b.wav" ).c_str(), ios::out | ios::binary);
    wave_ofstream_i .open((dir_laser + file + "_i.wav" ).c_str(), ios::out | ios::binary);
    //------------------------------------------------------------------------
    if(wave_in_header.num_channels == 8)
        wave_ofstream_LR.open((dir_audio + file + "_LR.wav").c_str(), ios::out | ios::binary);
    //------------------------------------------------------------------------
    if(    wave_ofstream_XY.is_open()
        && wave_ofstream_r .is_open()
        && wave_ofstream_g .is_open()
        && wave_ofstream_b .is_open()
        && wave_ofstream_i .is_open()
        && (    wave_ofstream_LR.is_open()
             || wave_in_header.num_channels != 8
           )
      )
    {
        split_wave_XY_r_g_b_i_LR(wave_in_header  ,
                                 wave_ifstream   ,
                                 wave_ofstream_XY,
                                 wave_ofstream_r ,
                                 wave_ofstream_g ,
                                 wave_ofstream_b ,
                                 wave_ofstream_i ,
                                 wave_ofstream_LR
                                );
        //--------------------------------------------------------------------
        wave_ifstream   .close();
        wave_ofstream_XY.close();
        wave_ofstream_r .close();
        wave_ofstream_g .close();
        wave_ofstream_b .close();
        wave_ofstream_i .close();
        if(wave_in_header.num_channels == 8)
            wave_ofstream_LR.close();
    }
    else
        return false;
    return true;
}

//############################################################################
bool LaserBoy_space::join_XY_r_g_b_i_waves_prep(   string file_XY
                                                 , string file_r
                                                 , string file_g
                                                 , string file_b
                                                 , string file_i
                                                 , string file_out
                                               )
{
    ifstream    in_XY,
                in_r,
                in_g,
                in_b,
                in_i;
    //------------------------------------------------------------------------
    in_XY.open(file_XY.c_str(), ios::in | ios::binary);
    in_r .open(file_r .c_str(), ios::in | ios::binary);
    in_g .open(file_g .c_str(), ios::in | ios::binary);
    in_b .open(file_b .c_str(), ios::in | ios::binary);
    in_i .open(file_i .c_str(), ios::in | ios::binary);
    //------------------------------------------------------------------------
    if(    in_XY.is_open()
        && in_r .is_open()
        && in_g .is_open()
        && in_b .is_open()
        && in_i .is_open()
      )
    {
        ofstream out;
        out.open(file_out.c_str(), ios::out | ios::binary);
        //--------------------------------------------------------------------
        join_XY_r_g_b_i_waves(in_XY, in_r, in_g, in_b, in_i, out);
        //--------------------------------------------------------------------
        in_XY.close();
        in_r .close();
        in_g .close();
        in_b .close();
        in_i .close();
        out  .close();
    }
    //------------------------------------------------------------------------
    else
        return false;
    return true;
}

//############################################################################
bool LaserBoy_space::join_XY_rg_b_waves_prep(   string file_XY
                                              , string file_rg
                                              , string file_b
                                              , string file_out
                                            )
{
    ifstream    in_XY,
                in_rg,
                in_b;
    //------------------------------------------------------------------------
    in_XY.open(file_XY.c_str(), ios::in | ios::binary);
    in_rg.open(file_rg.c_str(), ios::in | ios::binary);
    in_b .open(file_b .c_str(), ios::in | ios::binary);
    //------------------------------------------------------------------------
    if(    in_XY.is_open()
        && in_rg.is_open()
        && in_b .is_open()
      )
    {
        ofstream out;
        out.open(file_out.c_str(), ios::out | ios::binary);
        //--------------------------------------------------------------------
        join_XY_rg_b_waves(in_XY, in_rg, in_b, out);
        //--------------------------------------------------------------------
        in_XY.close();
        in_rg.close();
        in_b .close();
        out  .close();
    }
    //------------------------------------------------------------------------
    else
        return false;
    return true;
}

//############################################################################
bool LaserBoy_space::join_XY_i_LR_waves_prep(   string file_XY
                                              , string file_i
                                              , string file_LR
                                              , string file_out
                                            )
{
    ifstream in_XY,
             in_i ,
             in_LR;
    //------------------------------------------------------------------------
    in_XY.open(file_XY.c_str(), ios::in | ios::binary);
    in_i .open(file_i .c_str(), ios::in | ios::binary);
    in_LR.open(file_LR.c_str(), ios::in | ios::binary);
    //------------------------------------------------------------------------
    if(    in_XY.is_open()
        && in_i .is_open()
        && in_LR.is_open()
      )
    {
        ofstream out;
        //--------------------------------------------------------------------
        out.open(file_out.c_str(), ios::out | ios::binary);
        //--------------------------------------------------------------------
        join_XY_i_LR_waves(in_XY, in_i, in_LR, out);
        //--------------------------------------------------------------------
        in_XY.close();
        in_i .close();
        in_LR.close();
        out  .close();
    }
    else
        return false;
    return true;
}

//############################################################################
bool LaserBoy_space::add_audio_to_wave_prep(   string file_laser
                                             , string file_audio
                                             , string file_out
                                           )
{
    ifstream  in_laser,
              in_audio;
    //------------------------------------------------------------------------
    if(file_audio == "_0")
    {
        in_laser.open(file_laser.c_str(), ios::in | ios::binary);
        if(in_laser.is_open())
        {
            LaserBoy_wave_header wave_in_header_laser(in_laser);
            if(wave_in_header_laser.version != "!LaserBoy!") // It is a LaserBoy file!
            {
                ofstream out;
                out.open(file_out.c_str(), ios::out | ios::binary);
                //------------------------------------------------------------
                add_silence_to_wave(   wave_in_header_laser
                                     , in_laser
                                     , out
                                   );
                //------------------------------------------------------------
                in_laser.close();
                out.close();
                return true;
            } // end if(wave_in_header_laser.version != "!LaserBoy!")
            else
            {
                in_laser.close();
                return false;
            } // end else of if(wave_in_header_laser.version != "!LaserBoy!")
            in_laser.close();
        } // end if(in_laser.is_open())
        //--------------------------------------------------------------------
        else
            return false;
        //--------------------------------------------------------------------
    } // end if(file_audio == "_0")
    else
    {
        in_laser.open(file_laser.c_str(), ios::in | ios::binary);
        in_audio.open(file_audio.c_str(), ios::in | ios::binary);
        if(    in_laser.is_open()
            && in_audio.is_open()
          )
        {
            LaserBoy_wave_header wave_in_header_laser(in_laser),
                                 wave_in_header_audio(in_audio);

            if(wave_in_header_laser.version != "!LaserBoy!") // it is a LaserBoy wave!
            {
                ofstream             out;
                LaserBoy_wave_header wave_out_header(wave_in_header_laser, 0); // 8ch.
                //------------------------------------------------------------
                out.open(file_out.c_str(), ios::out | ios::binary);
                //------------------------------------------------------------
                add_audio_to_wave(   wave_in_header_laser
                                   , wave_in_header_audio
                                   , in_laser
                                   , in_audio
                                   , out
                                 );
                //------------------------------------------------------------
                in_laser.close();
                in_audio.close();
                out.close();
                return true;
            } // end if(wave_in_header_laser.version != "!LaserBoy!")
            else
            {
                in_laser.close();
                in_audio.close();
                return false;
            } // end else of if(wave_in_header_laser.version != "!LaserBoy!")
        } // end if(in_laser.is_open() && in_audio.is_open())
        //--------------------------------------------------------------------
        else
            return false;
        //--------------------------------------------------------------------
    } // end else of if(file_audio == "_0")
    return true;
}

//############################################################################
void LaserBoy_space::apply_wave_offsets(   ifstream& wave_ifstream
                                         , ofstream& wave_ofstream
                                         , LaserBoy_wave_header wave_in_header
                                       )
{
    int                            i                     ,
                                   span                  ,
                                   last                  ,
                                   offset_in_max         ,
                                   offset_in_min         ,
                                   offset_applied_max    ,
                                   offset_applied_min    ,
                                   offset_applied     [8];
    bool                           inverted       = false;
    LaserBoy_wave_sample_pointer   roll_over             ;
    LaserBoy_wave_sample_pointer  *sample_window         ;
    LaserBoy_wave_sample           wave_sample(wave_in_header.num_channels);
    LaserBoy_wave_header           wave_out_header(wave_in_header);
    //------------------------------------------------------------------------
    wave_out_header.LaserBoy_wave_mode |= LASERBOY_WAVE_OFFSETS;
    //------------------------------------------------------------------------
    for(i = 0; i < wave_in_header.num_channels; i++)
        offset_applied[i] = wave_in_header.offset[i] - wave_offsets[i];
    //------------------------------------------------------------------------
    offset_in_min      = wave_offsets  [0];
    offset_in_max      = wave_offsets  [0];
    offset_applied_min = offset_applied[0];
    offset_applied_max = offset_applied[0];
    //------------------------------------------------------------------------
    for(i = 0; i < wave_in_header.num_channels; i++)
    {
        if(wave_offsets  [i] > offset_in_max     ) offset_in_max      = wave_offsets  [i];
        if(wave_offsets  [i] < offset_in_min     ) offset_in_min      = wave_offsets  [i];
        if(offset_applied[i] > offset_applied_max) offset_applied_max = offset_applied[i];
        if(offset_applied[i] < offset_applied_min) offset_applied_min = offset_applied[i];
    }
    //------------------------------------------------------------------------
    span = (offset_applied_max - offset_applied_min) + 1;
    last = span - 1;
    //------------------------------------------------------------------------
    for(i = 0; i < wave_in_header.num_channels; i++)
    {
        wave_out_header.offset[i] = -(offset_in_max - wave_offsets[i]);
        offset_applied[i] = offset_applied_max - offset_applied[i];
    }
    //------------------------------------------------------------------------
    sample_window = new LaserBoy_wave_sample_pointer[span];
    for(i = 0; i < span; i++)
        sample_window[i] = new LaserBoy_wave_sample(wave_in_header.num_channels); // span element array of (6 or 8 channel LaserBoy_wave_samples)
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream); // take up the space!
    //------------------------------------------------------------------------
    while(true)
    {
        if(last) // span > 1
        {
            roll_over = sample_window[0];
            for(i = 0; i < last; i++)
                sample_window[i] = sample_window[i + 1];
            sample_window[last] = roll_over;
        }

        if(!sample_window[last]->from_ifstream_wave(wave_ifstream))
            break;

        for(i = 0; i < wave_in_header.num_channels; i++) // each channel
            wave_sample.channel[i] = sample_window[offset_applied[i]]->channel[i];
        wave_sample.to_ofstream_wave(wave_ofstream, wave_out_header); // write it to disk
    }
    delete sample_window;
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void LaserBoy_space::overwirte_wave_offsets(ifstream& wave_ifstream,
                                            ofstream& wave_ofstream,
                                            LaserBoy_wave_header wave_in_header
                                           )
{
    int                   i;
    LaserBoy_wave_sample  wave_sample(wave_in_header.num_channels);
    LaserBoy_wave_header  wave_out_header(wave_in_header);
    //------------------------------------------------------------------------
    for(i = 0; i < wave_in_header.num_channels; i++)
        wave_out_header.offset[i] = wave_offsets[i];
    wave_out_header.LaserBoy_wave_mode |= LASERBOY_WAVE_OFFSETS;
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream); // take up the space!
    //------------------------------------------------------------------------
    while(wave_sample.from_ifstream_wave(wave_ifstream))
        wave_sample.to_ofstream_wave(wave_ofstream, wave_out_header);
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void LaserBoy_space::invert_wave(ifstream& wave_ifstream,
                                 ofstream& wave_ofstream,
                                 LaserBoy_wave_header wave_in_header
                                )
{
    LaserBoy_wave_header wave_out_header(wave_in_header);
    LaserBoy_wave_sample wave_sample    (wave_in_header.num_channels);
    //------------------------------------------------------------------------
    if(wave_in_header.LaserBoy_wave_mode & LASERBOY_WAVE_POSITIVE)
        wave_out_header.LaserBoy_wave_mode &= ~LASERBOY_WAVE_POSITIVE;
    else
        wave_out_header.LaserBoy_wave_mode |= LASERBOY_WAVE_POSITIVE;
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream); // take up the space!
    //------------------------------------------------------------------------
    while(wave_sample.from_ifstream_wave_inverted(wave_ifstream))
        wave_sample.to_ofstream_wave(wave_ofstream, wave_out_header);
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void LaserBoy_space::invert_signals(ifstream& wave_ifstream,
                                    ofstream& wave_ofstream,
                                    LaserBoy_wave_header wave_in_header
                                   )
{
    int i;
    //------------------------------------------------------------------------
    wave_in_header.LaserBoy_wave_mode |= LASERBOY_WAVE_SIGNAL_MATRIX;
    LaserBoy_wave_header wave_out_header(wave_in_header);
    LaserBoy_wave_sample wave_sample    (wave_in_header.num_channels);
    //------------------------------------------------------------------------
    for(i = 0; i < wave_out_header.num_channels; i++)
        if(signal_polarity[i] == 1)
            wave_out_header.signal_id[i] = -wave_out_header.signal_id[i];
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream); // take up the space!
    //------------------------------------------------------------------------
    while(wave_sample.from_ifstream_wave_polarity(wave_ifstream, signal_polarity))
        wave_sample.to_ofstream_wave(wave_ofstream, wave_out_header);
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void LaserBoy_space::clear_wave_polarity_list(ifstream& wave_ifstream,
                                              ofstream& wave_ofstream,
                                              LaserBoy_wave_header wave_in_header
                                             )
{
    int i;
    //------------------------------------------------------------------------
    wave_in_header.LaserBoy_wave_mode |= LASERBOY_WAVE_SIGNAL_MATRIX;
    LaserBoy_wave_header wave_out_header(wave_in_header);
    //------------------------------------------------------------------------
    for(i = 0; i < wave_out_header.num_channels; i++)
        wave_out_header.signal_id[i] = abs(wave_out_header.signal_id[i]);
    //------------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(wave_ofstream); // that's it!
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void LaserBoy_space::split_wave_XY_r_g_b_i_LR(LaserBoy_wave_header wave_in_header,
                                              ifstream&       wave_ifstream,
                                              ofstream&       wave_ofstream_XY,
                                              ofstream&       wave_ofstream_r ,
                                              ofstream&       wave_ofstream_g ,
                                              ofstream&       wave_ofstream_b ,
                                              ofstream&       wave_ofstream_i ,
                                              ofstream&       wave_ofstream_LR
                                             )
{
    LaserBoy_wave_header  wave_out_header_XY(wave_in_header.sample_rate, LASERBOY_WAVE_NO_MODE, 2),
                          wave_out_header_r (wave_in_header.sample_rate, LASERBOY_WAVE_NO_MODE, 1),
                          wave_out_header_g (wave_in_header.sample_rate, LASERBOY_WAVE_NO_MODE, 1),
                          wave_out_header_b (wave_in_header.sample_rate, LASERBOY_WAVE_NO_MODE, 1),
                          wave_out_header_i (wave_in_header.sample_rate, LASERBOY_WAVE_NO_MODE, 1),
                          wave_out_header_LR(wave_in_header.sample_rate, LASERBOY_WAVE_NO_MODE, 2);

    LaserBoy_wave_sample  stereo_sample(2),
                          mono_sample  (1);
    //------------------------------------------------------------------------
    LaserBoy_wave_sample wave_sample(wave_in_header.num_channels);
    //------------------------------------------------------------------------
    while(wave_sample.from_ifstream_wave(wave_ifstream))
    {
        stereo_sample.channel[0] = wave_sample.channel[0];
        stereo_sample.channel[1] = wave_sample.channel[1];
        stereo_sample.to_ofstream_wave(wave_ofstream_XY, wave_out_header_XY);

        mono_sample.channel[0]   = wave_sample.channel[2];
        mono_sample.to_ofstream_wave(wave_ofstream_r, wave_out_header_r);

        mono_sample.channel[0]   = wave_sample.channel[3];
        mono_sample.to_ofstream_wave(wave_ofstream_g, wave_out_header_g);

        mono_sample.channel[0]   = wave_sample.channel[4];
        mono_sample.to_ofstream_wave(wave_ofstream_b, wave_out_header_b);

        mono_sample.channel[0]   = wave_sample.channel[5];
        mono_sample.to_ofstream_wave(wave_ofstream_i, wave_out_header_i);
        //--------------------------------------------------------------------
        if(wave_in_header.num_channels == 8)
        {
            stereo_sample.channel[0] = wave_sample.channel[6];
            stereo_sample.channel[1] = wave_sample.channel[7];
            stereo_sample.to_ofstream_wave(wave_ofstream_LR, wave_out_header_LR);
        }
    }
    //------------------------------------------------------------------------
    wave_out_header_XY.to_ofstream_wave(wave_ofstream_XY);
    wave_out_header_r .to_ofstream_wave(wave_ofstream_r );
    wave_out_header_g .to_ofstream_wave(wave_ofstream_g );
    wave_out_header_b .to_ofstream_wave(wave_ofstream_b );
    wave_out_header_i .to_ofstream_wave(wave_ofstream_i );
    //------------------------------------------------------------------------
    if(wave_in_header.num_channels == 8)
        wave_out_header_LR.to_ofstream_wave(wave_ofstream_LR);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void LaserBoy_space::join_XY_r_g_b_i_waves(ifstream&     in_XY,
                                           ifstream&     in_r,
                                           ifstream&     in_g,
                                           ifstream&     in_b,
                                           ifstream&     in_i,
                                           ofstream&     out
                                          )
{
    LaserBoy_wave_sample sample_XYrgbi(6),
                         sample_XY    (2),
                         sample_r     (1),
                         sample_g     (1),
                         sample_b     (1),
                         sample_i     (1);
    //--------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header_XY(in_XY),
                         wave_in_header_r (in_r ),
                         wave_in_header_g (in_g ),
                         wave_in_header_b (in_b ),
                         wave_in_header_i (in_i );
    //--------------------------------------------------------------------
    LaserBoy_wave_header wave_out_header  (  wave_in_header_XY.sample_rate,
                                             LASERBOY_WAVE_POSITIVE
                                           | LASERBOY_WAVE_OFFSETS
                                           | LASERBOY_WAVE_SIGNAL_MATRIX
                                           | LASERBOY_SIGNAL_BIT_RESOLUTION,
                                           6
                                          );
    //--------------------------------------------------------------------
    wave_out_header.offset    [0] = 0;
    wave_out_header.offset    [1] = 0;
    wave_out_header.offset    [2] = 0;
    wave_out_header.offset    [3] = 0;
    wave_out_header.offset    [4] = 0;
    wave_out_header.offset    [5] = 0;

    wave_out_header.signal_id [0] = LASERBOY_SIGNAL_X_POSITION  ;
    wave_out_header.signal_id [1] = LASERBOY_SIGNAL_Y_POSITION  ;
    wave_out_header.signal_id [2] = LASERBOY_SIGNAL_RED_ANALOG  ;
    wave_out_header.signal_id [3] = LASERBOY_SIGNAL_GREEN_ANALOG;
    wave_out_header.signal_id [4] = LASERBOY_SIGNAL_BLUE_ANALOG ;
    wave_out_header.signal_id [5] = LASERBOY_SIGNAL_UNDEFINED   ;

    wave_out_header.LSB_tag   [0] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [1] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [2] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [3] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [4] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [5] = LASERBOY_LSB_NOT_USED;

    wave_out_header.resolution[0] = 16;
    wave_out_header.resolution[1] = 16;
    wave_out_header.resolution[2] =  8;
    wave_out_header.resolution[3] =  8;
    wave_out_header.resolution[4] =  8;
    wave_out_header.resolution[5] = 16;
    //--------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out); // take up the space!
    //--------------------------------------------------------------------
    while(    sample_XY.from_ifstream_wave(in_XY)
           && sample_r .from_ifstream_wave(in_r )
           && sample_g .from_ifstream_wave(in_g )
           && sample_b .from_ifstream_wave(in_b )
           && sample_i .from_ifstream_wave(in_i )
         )
    {
        sample_XYrgbi =   sample_XY
                        + sample_r
                        + sample_g
                        + sample_b
                        + sample_i; // isn't that neat!

        sample_XYrgbi.to_ofstream_wave(out, wave_out_header);
    }
    //--------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out);
    return;
}

//############################################################################
void LaserBoy_space::join_XY_rg_b_waves(ifstream&     in_XY,
                                        ifstream&     in_rg,
                                        ifstream&     in_b,
                                        ofstream&     out
                                       )
{
    LaserBoy_wave_sample sample_XYrgb (6),
                         sample_XY    (2),
                         sample_rg    (2),
                         sample_b     (1),
                         sample_0     (1); // silent sample
    //--------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header_XY(in_XY),
                         wave_in_header_rg(in_rg),
                         wave_in_header_b (in_b );
    //--------------------------------------------------------------------
    LaserBoy_wave_header wave_out_header  (  wave_in_header_XY.sample_rate,
                                             LASERBOY_WAVE_POSITIVE
                                           | LASERBOY_WAVE_OFFSETS
                                           | LASERBOY_WAVE_SIGNAL_MATRIX
                                           | LASERBOY_SIGNAL_BIT_RESOLUTION,
                                           6
                                          );
    //--------------------------------------------------------------------
    wave_out_header.offset    [0] = 0;
    wave_out_header.offset    [1] = 0;
    wave_out_header.offset    [2] = 0;
    wave_out_header.offset    [3] = 0;
    wave_out_header.offset    [4] = 0;
    wave_out_header.offset    [5] = 0;

    wave_out_header.signal_id [0] = LASERBOY_SIGNAL_X_POSITION  ;
    wave_out_header.signal_id [1] = LASERBOY_SIGNAL_Y_POSITION  ;
    wave_out_header.signal_id [2] = LASERBOY_SIGNAL_RED_ANALOG  ;
    wave_out_header.signal_id [3] = LASERBOY_SIGNAL_GREEN_ANALOG;
    wave_out_header.signal_id [4] = LASERBOY_SIGNAL_BLUE_ANALOG ;
    wave_out_header.signal_id [5] = LASERBOY_SIGNAL_UNDEFINED   ;

    wave_out_header.LSB_tag   [0] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [1] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [2] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [3] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [4] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [5] = LASERBOY_LSB_NOT_USED;

    wave_out_header.resolution[0] = 16;
    wave_out_header.resolution[1] = 16;
    wave_out_header.resolution[2] =  8;
    wave_out_header.resolution[3] =  8;
    wave_out_header.resolution[4] =  8;
    wave_out_header.resolution[5] = 16;
    //--------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out); // take up the space!
    //--------------------------------------------------------------------
    while(    sample_XY.from_ifstream_wave(in_XY)
           && sample_rg.from_ifstream_wave(in_rg)
           && sample_b .from_ifstream_wave(in_b )
         )
    {
        sample_XYrgb =   sample_XY
                       + sample_rg
                       + sample_b
                       + sample_0; // isn't that neat!

        sample_XYrgb.to_ofstream_wave(out, wave_out_header);
    }
    //--------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out);
    return;
}

//############################################################################
void LaserBoy_space::join_XY_i_LR_waves(ifstream&     in_XY,
                                        ifstream&     in_i ,
                                        ifstream&     in_LR,
                                        ofstream&     out
                                       )
{
    LaserBoy_wave_sample sample_XYikLR(6),
                         sample_XY    (2),
                         sample_i     (1),
                         sample_k     (1),
                         sample_LR    (2);
    //--------------------------------------------------------------------
    LaserBoy_wave_header wave_in_header_XY(in_XY),
                         wave_in_header_i (in_i ),
                         wave_in_header_LR(in_LR);
    //--------------------------------------------------------------------
    LaserBoy_wave_header wave_out_header  (  wave_in_header_XY.sample_rate,
                                             LASERBOY_WAVE_POSITIVE
                                           | LASERBOY_WAVE_OFFSETS
                                           | LASERBOY_WAVE_SIGNAL_MATRIX
                                           | LASERBOY_SIGNAL_BIT_RESOLUTION,
                                           6
                                          );
    //--------------------------------------------------------------------
    wave_out_header.offset    [0] = 0;
    wave_out_header.offset    [1] = 0;
    wave_out_header.offset    [2] = 0;
    wave_out_header.offset    [3] = 0;
    wave_out_header.offset    [4] = 0;
    wave_out_header.offset    [5] = 0;

    wave_out_header.signal_id [0] = LASERBOY_SIGNAL_X_POSITION     ;
    wave_out_header.signal_id [1] = LASERBOY_SIGNAL_Y_POSITION     ;
    wave_out_header.signal_id [2] = LASERBOY_SIGNAL_MONO_AVG_ANALOG;
    wave_out_header.signal_id [3] = LASERBOY_SIGNAL_MONO_TTL       ;
    wave_out_header.signal_id [4] = LASERBOY_SIGNAL_AUDIO_LEFT     ;
    wave_out_header.signal_id [5] = LASERBOY_SIGNAL_AUDIO_RIGHT    ;

    wave_out_header.LSB_tag   [0] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [1] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [2] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [3] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [4] = LASERBOY_LSB_NOT_USED;
    wave_out_header.LSB_tag   [5] = LASERBOY_LSB_NOT_USED;

    wave_out_header.resolution[0] = 16;
    wave_out_header.resolution[1] = 16;
    wave_out_header.resolution[2] =  8;
    wave_out_header.resolution[3] =  8;
    wave_out_header.resolution[4] = 16;
    wave_out_header.resolution[5] = 16;
    //--------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out); // take up the space!
    //--------------------------------------------------------------------
    while(    sample_XY.from_ifstream_wave(in_XY)
           && sample_i .from_ifstream_wave(in_i )
           && sample_LR.from_ifstream_wave(in_LR)
         )
    {
        sample_k.channel[0] = (   (sample_i.channel[0] > (LASERBOY_MAX_COLOR_SHORT / 4))
                                ? (LASERBOY_MAX_COLOR_SHORT)
                                : (0)
                              );

        sample_XYikLR =   sample_XY
                        + sample_i
                        + sample_k
                        + sample_LR;

        sample_XYikLR.to_ofstream_wave(out, wave_out_header);
    }
    //--------------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void LaserBoy_space::add_audio_to_wave(LaserBoy_wave_header wave_in_header_laser,
                                       LaserBoy_wave_header wave_in_header_audio,
                                       ifstream&            in_laser,
                                       ifstream&            in_audio,
                                       ofstream&            out
                                      )
{
    LaserBoy_wave_sample sample_laser      (6), // 6 chennels
                         sample_audio      (2), // 2 chennels
                         sample_laser_audio(8); // 8 chennels

    LaserBoy_wave_header wave_out_header   (wave_in_header_laser, 0); // 8ch.
    //----------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out); // take up the space!
    while(    sample_laser.from_ifstream_wave(in_laser)
           && sample_audio.from_ifstream_wave(in_audio)
         )
    {
        sample_laser_audio = sample_laser + sample_audio; // isn't that neat!
        sample_laser_audio.to_ofstream_wave(out, wave_out_header);
    }
    //----------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out);
    return;
}

//############################################################################
void LaserBoy_space::add_silence_to_wave(LaserBoy_wave_header wave_in_header_laser,
                                         ifstream&            in_laser,
                                         ofstream&            out
                                        )
{
    LaserBoy_wave_sample sample_laser      (6), // 6 chennels
                         sample_audio      (2), // 2 chennels (zero)
                         sample_laser_audio(8); // 8 chennels

    LaserBoy_wave_header wave_out_header(wave_in_header_laser, 0); // 8ch.
    //------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out); // take up the space!
    while(sample_laser.from_ifstream_wave(in_laser))
    {
        sample_laser_audio = sample_laser + sample_audio; // isn't that neat!
        sample_laser_audio.to_ofstream_wave(out, wave_out_header);
    }
    //------------------------------------------------------------
    wave_out_header.to_ofstream_wave(out);
    return;
}

//############################################################################
string LaserBoy_space::LaserBoy_wave_signal_id_to_name(const short& signal_id) const
{
    switch(abs(signal_id))
    {
        default:
        case LASERBOY_SIGNAL_NO_SIGNAL:
             return "NO SIGNAL   ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_UNDEFINED:
             return "UNDEF SIGNAL";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_X_POSITION:
             return "X POSITION  ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_Y_POSITION:
             return "Y POSITION  ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_Z_POSITION:
             return "Z POSITION  ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_DX_POSITION:
             return "DX POSITION ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_DY_POSITION:
             return "DY POSITION ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_X2_POSITION:
             return "DX POSITION ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_Y2_POSITION:
             return "DY POSITION ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_BEAM_WIDTH:
             return "BEAM WIDTH  ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_RED_TTL:
             return "RED   TTL   ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_RED_ANALOG:
             return "RED   ANALOG";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_YELLOW_TTL:
             return "YELLOW TTL  ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_YELLOW_ANALOG:
             return "YELLOW ANALG";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_GREEN_TTL:
             return "GREEN TTL   ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_GREEN_ANALOG:
             return "GREEN ANALOG";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_BLUE_TTL:
             return "BLUE  TTL   ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_BLUE_ANALOG:
             return "BLUE  ANALOG";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_VIOLET_TTL:
             return "VIOLET TTL  ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_VIOLET_ANALOG:
             return "VIOLET ANALG";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_MONO_TTL:
             return "MONO  TTL   ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_MONO_AVG_ANALOG:
             return "MONO AVG ANL";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_MONO_OR_ANALOG:
             return "MONO OR ANLG";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_MONO_WEIGHTED_ANALOG:
             return "MONO WTD ANL";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_SMTPE:
             return "AUDIO SMTPE ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_MONO:
             return "AUDIO MONO  ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_LEFT:
             return "AUDIO LEFT  ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_RIGHT:
             return "AUDIO RIGHT ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_CENTER:
             return "AUDIO CENTER";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_SUB:
             return "AUDIO SUB   ";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_S_LEFT:
             return "AUDIO S LEFT";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_S_RIGHT:
             return "AUDIO S RGHT";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_R_LEFT:
             return "AUDIO R LEFT";
        //--------------------------------------------------------------------
        case LASERBOY_SIGNAL_AUDIO_R_RIGHT:
             return "AUDIO R RGHT";
        //--------------------------------------------------------------------
    }
    return "";
}

//############################################################################
string LaserBoy_space::LaserBoy_wave_LSB_tag_to_name(const short& LSB_tag) const
{
    switch(LSB_tag)
    {
        default:
        case LASERBOY_LSB_NOT_USED:
             return "            ";
             break;
        //--------------------------------------------------------------------
        case LASERBOY_LSB_BLANKING:
             return "BLANKING    ";
             break;
        //--------------------------------------------------------------------
        case LASERBOY_LSB_END_OF_FRAME:
             return "END OF FRAME";
             break;
        //--------------------------------------------------------------------
        case LASERBOY_LSB_UNIQUE_FRAME:
             return "UNIQUE FRAME";
             break;
        //--------------------------------------------------------------------
        case LASERBOY_LSB_UNIQUE_VERTEX:
             return "UNIQUE VERTX";
             break;
        //--------------------------------------------------------------------
    }
    return "";
}


//############################################################################
void LaserBoy_space::tell(LaserBoy_wave_header header, string label) const
{
    cout << "----------------------------------------------------------" << endl;
    cout << label << endl;
    cout << "             num_samples : " << header.num_samples     << endl;
    cout << "             sample_rate : " << header.sample_rate     << endl;
    cout << "            LaserBoy_wave_mode : ";
    //------------------------------------------------------------------------
    if(header.LaserBoy_wave_mode != LASERBOY_WAVE_NO_MODE)
    {
        if(header.LaserBoy_wave_mode & LASERBOY_WAVE_POSITIVE        )
            cout << "LASERBOY_WAVE_POSITIVE" << endl;
        else
            cout << "LASERBOY_WAVE_NEGATIVE" << endl;

        if(header.LaserBoy_wave_mode & LASERBOY_WAVE_END_OF_FRAME    )
            cout << "                           LASERBOY_WAVE_END_OF_FRAME"     << endl;
        if(header.LaserBoy_wave_mode & LASERBOY_WAVE_UNIQUE_FRAME    )
            cout << "                           LASERBOY_WAVE_UNIQUE_FRAME"     << endl;
        if(header.LaserBoy_wave_mode & LASERBOY_WAVE_UNIQUE_VERTEX    )
            cout << "                           LASERBOY_WAVE_UNIQUE_VERTEX"     << endl;
        if(header.LaserBoy_wave_mode & LASERBOY_WAVE_OFFSETS         )
            cout << "                           LASERBOY_WAVE_OFFSETS"          << endl;
        if(header.LaserBoy_wave_mode & LASERBOY_WAVE_OPTIMIZED       )
            cout << "                           LASERBOY_WAVE_OPTIMIZED"        << endl;
        if(header.LaserBoy_wave_mode & LASERBOY_WAVE_SIGNAL_MATRIX   )
            cout << "                           LASERBOY_WAVE_SIGNAL_MATRIX"    << endl;
        if(header.LaserBoy_wave_mode & LASERBOY_SIGNAL_BIT_RESOLUTION)
            cout << "                           LASERBOY_SIGNAL_BIT_RESOLUTION" << endl;
        //--------------------------------------------------------------------
        cout << "            num_channels : " << header.num_channels    << endl;
        cout << "         bits_per_sample : " << header.bits_per_sample << endl;
        cout << "                 version : " << header.version         << endl << endl;
        //--------------------------------------------------------------------
        cout << " ch. :offset: bits :     : signal name  : LSB tag"          << endl;
        cout << "-----:------:------:-----:--------------:-----------------" << endl;
        for(int i = 0; i < header.num_channels; i++)
        {
            cout << setw(4)
                 << i
                 << " : ";
            if(header.LaserBoy_wave_mode & LASERBOY_WAVE_OFFSETS)
                 cout << setw(4) << (int)header.offset[i];
            else
                 cout << " na ";
            cout << " : ";

            if(header.LaserBoy_wave_mode & LASERBOY_SIGNAL_BIT_RESOLUTION)
                 cout << setw(4) << (int)header.resolution[i];
            else
                 cout << "    ";

            if(header.LaserBoy_wave_mode & LASERBOY_WAVE_SIGNAL_MATRIX)
                cout << " : "
                     << ((header.signal_id[i] >= 0) ? ("pos") : ("neg"))
                     << " : "
                     << LaserBoy_wave_signal_id_to_name(header.signal_id[i])
                     << " : "
                     << LaserBoy_wave_LSB_tag_to_name(header.LSB_tag[i])
                     << endl;
            else
                cout << endl;
        }
        //--------------------------------------------------------------------
        if(header.LaserBoy_wave_mode & LASERBOY_WAVE_OPTIMIZED)
        {
            cout << endl;
            cout << "    lit_dwell_overhang : " << header.parms.lit_dwell_overhang     << endl;
            cout << "         lit_delta_max : " << header.parms.lit_delta_max          << endl;
            cout << "       blank_delta_max : " << header.parms.blank_delta_max        << endl;
            cout << "    max_dwell_microsec : " << header.parms.max_dwell_microsec     << endl;
            cout << "insignificant_distance : " << header.parms.insignificant_distance << endl;
            cout << "   insignificant_angle : " << header.parms.insignificant_angle    << endl;
            cout << "     frames_per_second : " << header.parms.frames_per_second      << endl;
        }
    }
    else
    {
        cout << "LASERBOY_WAVE_NO_MODE" << endl;
    }
    cout << "----------------------------------------------------------" << endl;
    return;
}

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
